const GLib = imports.gi.GLib;
const Main = imports.ui.main;
const Config = imports.misc.config;
const Version = parseInt(Config.PACKAGE_VERSION.split('.')[0]);
const ImportClass = Version == 3 ? imports.ui.viewSelector.ViewSelector : imports.ui.overviewControls.ControlsManager;
const ShowAppsButton = Version == 3 ? Main.overview.viewSelector._showAppsButton : Main.overview.dash.showAppsButton;
const OverviewShowApps = Version == 3 ? Main.overview.viewSelector : Main.overview;
const MainOverview = Version == 3 ? Main.overview.viewSelector : Main.overview.dash;

let _signal = [];
let _function;

let _manager, _workspace, _monitor;

var _showAppsButtonChecked;

function hideApps()
{
    // hide the overview only if we're in application view
    if (ShowAppsButton.checked)
        Main.overview.hide();
}

function showApps()
{
    if (_workspace.list_windows()./*filter(window => window.get_monitor() == _monitor).*/length == 0)
        OverviewShowApps.showApps();
}

function disconnectWindowSignals()
{
    if (_signal['window-added'])
        _workspace.disconnect(_signal['window-added']);

    if (_signal['window-removed'])
        _workspace.disconnect(_signal['window-removed']);
}

function getWorkspace()
{
    _workspace = _manager.get_active_workspace();

    _signal['window-added'] = _workspace.connect('window-added', hideApps);
    _signal['window-removed'] = _workspace.connect('window-removed', showApps);
}

function checkWorkspace()
{
    disconnectWindowSignals();

    getWorkspace();

    if (!Main.overview.visible)
        showApps();
    else
        hideApps();
}

function overviewHidden()
{
    // don't show the apps view if we were already looking at it
    if (!_showAppsButtonChecked)
        showApps();
}

function animateFromOverview(callback)
{
    // the original function sets _showAppsButton.checked = false, so we need to copy it to a local variable first
    _showAppsButtonChecked = ShowAppsButton.checked;
    _function.apply(this, [callback]);
}

function init()
{
    _manager = global.screen;
    if (_manager == undefined)
        _manager = global.workspace_manager;

    _monitor = global.display.get_primary_monitor();

    _function = ImportClass.prototype.animateFromOverview;
}

function enable()
{
    ImportClass.prototype.animateFromOverview = animateFromOverview;

    getWorkspace();

    _signal['workspace-switched'] = _manager.connect('workspace-switched', checkWorkspace);
    _signal['overview-hidden'] = Main.overview.connect('hidden', overviewHidden);

    // shows applications at login, but waits for mainloop to turn idle first
    // .. and then waits 1 second
    GLib.idle_add(GLib.PRIORITY_LOW, () => {
        GLib.timeout_add_seconds(GLib.PRIORITY_LOW, 0, () => {
            if (!Main.overview.visible)
                showApps();
            else
            {
                ShowAppsButton.checked = true;
                MainOverview._onShowAppsButtonToggled();
            }
            return GLib.SOURCE_REMOVE;
        });
        return GLib.SOURCE_REMOVE;
    });
}

function disable()
{
    disconnectWindowSignals();

    Main.overview.disconnect(_signal['overview-hidden']);
    _manager.disconnect(_signal['workspace-switched']);

    ImportClass.prototype.animateFromOverview = _function;
}
